/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2012                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_off.h>
#include<vcg/complex/algorithms/update/topology.h>
#include<vcg/complex/algorithms/update/normal.h>
using namespace vcg;

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public UsedTypes<Use<MyVertex>   ::AsVertexType,
                                           Use<MyEdge>     ::AsEdgeType,
                                           Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public Vertex< MyUsedTypes, vertex::Coord3f, vertex::Normal3f,
    vertex::VFAdj,
                                              vertex::BitFlags  >{};
class MyFace    : public Face<   MyUsedTypes, face::VFAdj,
                                      face::VertexRef, face::BitFlags > {};
class MyEdge    : public Edge<   MyUsedTypes> {};

class MyMesh    : public tri::TriMesh< std::vector<MyVertex>,
                                       std::vector<MyFace> ,
                                       std::vector<MyEdge>  > {};

template <class MeshType>
void Smooth(MeshType &m)
{
  typedef typename MeshType::VertexType VertexType;
  typedef typename MeshType::FaceType FaceType;
  typedef typename MeshType::VertexIterator VertexIterator;

  tri::RequireVFAdjacency(m);
  tri::UpdateTopology<MeshType>::VertexFace(m);

  typename MeshType:: template PerVertexAttributeHandle <Point3f>  avgH =
      tri::Allocator<MeshType>:: template GetPerVertexAttribute<Point3f>(m,"avg");

  for(VertexIterator vi=m.vert.begin();vi!=m.vert.end();++vi)
  {
   std::vector<VertexType *> starVec;
   face::VVStarVF<FaceType>(&*vi,starVec);
   avgH[vi] = Point3f(0,0,0);

   for(size_t i=0;i<starVec.size();++i)
   {
     avgH[vi] +=starVec[i]->P();
   }
   avgH[vi] /= float(starVec.size());
  }

  for(VertexIterator vi=m.vert.begin();vi!=m.vert.end();++vi)
    vi->P() = avgH[vi];
}

int main( int argc, char **argv )
{
  if(argc<2)
  {
    printf("Usage trimesh_base <meshfilename.obj>\n");
    return -1;
  }
  /*!
    */
  MyMesh m;

  if(tri::io::ImporterOFF<MyMesh>::Open(m,argv[1])!=
     tri::io::ImporterOFF<MyMesh>::NoError)
  {
    printf("Error reading file  %s\n",argv[1]);
    exit(0);
  }

  for(int i=0;i<10;++i)
    Smooth(m);

  tri::io::ExporterOFF<MyMesh>::Save(m,"smooth.off");

  return 0;
}




